@@ -31,9 +31,7 @@ from __future__ import print_function
 
 import os
 import sys
-import tempfile
 import yaml
-import hashlib
 try:
     from urllib.request import urlopen
     from urllib.error import URLError
@@ -45,8 +43,10 @@ try:
 except ImportError:
     import pickle
 
+from .cache_tools import compute_filename_hash, PICKLE_CACHE_EXT, write_atomic, write_cache_file
 from .core import InvalidData, DownloadFailure, CachePermissionError
 from .gbpdistro_support import get_gbprepo_as_rosdep_data, download_gbpdistro_as_rosdep_data
+from .meta import MetaDatabase
 
 try:
     import urlparse
@@ -78,7 +78,6 @@ SOURCES_CACHE_DIR = 'sources.cache'
 CACHE_INDEX = 'index'
 
 # extension for binary cache
-PICKLE_CACHE_EXT = '.pickle'
 SOURCE_PATH_ENV = 'ROSDEP_SOURCE_PATH'
 
 
@@ -203,7 +202,7 @@ def cache_data_source_loader(sources_cache_dir, verbose=False):
             if verbose:
                 print('loading cached data source:\n\t%s\n\t%s' % (uri, filepath), file=sys.stderr)
             with open(filepath) as f:
-                rosdep_data = yaml.load(f.read())
+                rosdep_data = yaml.safe_load(f.read())
         else:
             rosdep_data = {}
         return CachedDataSource(type_, uri, tags, rosdep_data, origin=filepath)
@@ -485,6 +484,8 @@ def update_sources_list(sources_list_dir=None, sources_cache_dir=None,
 
     # Additional sources for ros distros
     # In compliance with REP137 and REP143
+    python_versions = {}
+
     print('Query rosdistro index %s' % get_index_url())
     for dist_name in sorted(get_index().distributions.keys()):
         distribution = get_index().distributions[dist_name]
@@ -495,12 +496,18 @@ def update_sources_list(sources_list_dir=None, sources_cache_dir=None,
         print('Add distro "%s"' % dist_name)
         rds = RosDistroSource(dist_name)
         rosdep_data = get_gbprepo_as_rosdep_data(dist_name)
+        # Store Python version from REP153
+        if distribution.get('python_version'):
+            python_versions[dist_name] = distribution.get('python_version')
         # dist_files can either be a string (single filename) or a list (list of filenames)
         dist_files = distribution['distribution']
         key = _generate_key_from_urls(dist_files)
         retval.append((rds, write_cache_file(sources_cache_dir, key, rosdep_data)))
         sources.append(rds)
 
+    # cache metadata that isn't a source list
+    MetaDatabase().set('ROS_PYTHON_VERSION', python_versions)
+
     # Create a combined index of *all* the sources.  We do all the
     # sources regardless of failures because a cache from a previous
     # attempt may still exist.  We have to do this cache index so that
@@ -540,68 +547,6 @@ def load_cached_sources_list(sources_cache_dir=None, verbose=False):
     return parse_sources_data(cache_data, origin=cache_index, model=model)
 
 
-def compute_filename_hash(key_filenames):
-    sha_hash = hashlib.sha1()
-    if isinstance(key_filenames, list):
-        for key in key_filenames:
-            sha_hash.update(key.encode())
-    else:
-        sha_hash.update(key_filenames.encode())
-    return sha_hash.hexdigest()
-
-
-def write_cache_file(source_cache_d, key_filenames, rosdep_data):
-    """
-    :param source_cache_d: directory to write cache file to
-    :param key_filenames: filename (or list of filenames) to be used in hashing
-    :param rosdep_data: dictionary of data to serialize as YAML
-    :returns: name of file where cache is stored
-    :raises: :exc:`OSError` if cannot write to cache file/directory
-    :raises: :exc:`IOError` if cannot write to cache file/directory
-    """
-    if not os.path.exists(source_cache_d):
-        os.makedirs(source_cache_d)
-    key_hash = compute_filename_hash(key_filenames)
-    filepath = os.path.join(source_cache_d, key_hash)
-    try:
-        write_atomic(filepath + PICKLE_CACHE_EXT, pickle.dumps(rosdep_data, 2), True)
-    except OSError as e:
-        raise CachePermissionError('Failed to write cache file: ' + str(e))
-    try:
-        os.unlink(filepath)
-    except OSError:
-        pass
-    return filepath
-
-
-def write_atomic(filepath, data, binary=False):
-    # write data to new file
-    fd, filepath_tmp = tempfile.mkstemp(prefix=os.path.basename(filepath) + '.tmp.', dir=os.path.dirname(filepath))
-
-    if (binary):
-        fmode = 'wb'
-    else:
-        fmode = 'w'
-
-    with os.fdopen(fd, fmode) as f:
-        f.write(data)
-        f.close()
-
-    try:
-        # switch file atomically (if supported)
-        os.rename(filepath_tmp, filepath)
-    except OSError:
-        # fall back to non-atomic operation
-        try:
-            os.unlink(filepath)
-        except OSError:
-            pass
-        try:
-            os.rename(filepath_tmp, filepath)
-        except OSError:
-            os.unlink(filepath_tmp)
-
-
 class SourcesListLoader(RosdepLoader):
     """
     SourcesList loader implements the general RosdepLoader API.  This
